from izzylib import izzylog
from izzylib.path import Path


class Connection:
	def __init__(self, db):
		self.db = db
		self.cfg = db.cfg
		self.sessions = {}
		self.conn = None

		self.connect()

		#if db.tables.empty:
			#with self.session as s:
				#db.load_tables(s.table_layout())


	@property
	def autocommit(self):
		return self.conn.autocommit


	@property
	def session(self):
		return self.cfg.session_class(self)


	def connect(self):
		if self.conn:
			return

		dbconfig = self.cfg.dbargs

		if self.cfg.type == 'sqlite':
			if self.cfg.autocommit:
				self.conn = self.cfg.module.connect(
					dbconfig['database'],
					isolation_level = None,
					**self.cfg.engine_args
				)

			else:
				self.conn = self.cfg.module.connect(
					dbconfig['database'],
					isolation_level = 'immediate',
					**self.cfg.engine_args
				)

		elif self.cfg.type == 'postgresql':
			if Path(self.cfg.host).exists():
				dbconfig['unix_sock'] = dbconfig.pop('host')

			dbconfig['user'] = dbconfig.pop('username')
			dbconfig['application_name'] = self.cfg.appname
			self.conn = self.cfg.module.connect(**dbconfig, **self.cfg.engine_args)

		else:
			self.conn = self.cfg.module.connect(**self.cfg.dbargs, **self.cfg.engine_args)

		try:
			self.conn.autocommit = self.cfg.autocommit

		except AttributeError:
			if self.cfg.module_name not in ['sqlite3']:
				izzylog.verbose('Module does not support autocommit:', self.cfg.module_name)

		return self.conn


	def disconnect(self):
		if not self.conn:
			return

		self.close_sessions()
		self.conn.close()
		self.conn = None


	def close_sessions(self):
		for session in self.sessions:
			self.close_session(session)


	def close_session(session):
		try: self.sessions.remove(session)
		except: pass

		session.close()

		if not len(self.sessions) and len(self.db.connections) > self.cfg.minconnections:
			self.disconnect()


	def cursor(self):
		if not self.conn:
			raise Exception('e')
		return self.conn.cursor()


	def dump_database(self, path='database.sql'):
		if self.cfg.type == 'sqlite':
			path = Path(path)

			with path.open('w') as fd:
				fd.write('\n\n'.join(list(self.conn.iterdump())[1:-1]))

		else:
			raise NotImplementedError('Only SQLite supported atm :/')


	def execute(self, string, *values):
		cursor = self.cursor()

		if values:
			cursor.execute(string, values)

		else:
			cursor.execute(string)

		data = cursor.fetchall()
		cursor.close()

		return data
