from .row import Row


class Result:
	def __init__(self, session):
		self.table = None
		self.session = session
		self.cursor = session.cursor

		try:
			self.keys = [desc[0] for desc in session.cursor.description]

		except TypeError:
			self.keys = []


	def __iter__(self):
		yield from self.all_iter()


	@property
	def row_class(self):
		return self.session.db.get_row_class(self.table)


	@property
	def last_row_id(self):
		if self.session.cfg.type == 'postgresql':
			try:
				return self.one().id

			except:
				return None

		return self.cursor.lastrowid


	@property
	def row_count(self):
		return self.cursor.rowcount


	def set_table(self, table):
		self.table = table


	def one(self):
		data = self.cursor.fetchone()

		if not data:
			return

		return self.row_class(
			self.session,
			self.table,
			{self.keys[idx]: value for idx, value in enumerate(data)},
		)


	def all(self):
		return [row for row in self.all_iter()]


	def all_iter(self):
		for row in self.cursor:
			yield self.row_class(self.session, self.table,
				{self.keys[idx]: value for idx, value in enumerate(row)}
			)
